﻿using System;
using System.Windows.Forms;
using System.IO.Ports;	//SerialPort

namespace Seg7ClockControler
{
	public partial class FormMain : Form
	{
		private const string
			APP_DATE	= "2011/03",
			APP_VER		= "ver 1.01",
			APP_AUTHOR	= "『昼夜逆転』工作室 J.Sugita";
		private static readonly string
			APP_ABOUT	= APP_DATE + "  " + APP_VER + Environment.NewLine + APP_AUTHOR;
		
		private CdcIO cdcio;
		private Seg7Clock clockDevice;
		private bool isConnect;
		
		//コンストラクタ
		public FormMain()
		{
			InitializeComponent();
			lblVersion.Text = APP_VER;
			
			cdcio = new CdcIO();
			clockDevice = new Seg7Clock();
			isConnect = false;

			//日時のリストボックスのItemsを登録し、描画更新を開始する
			setDateControlValue();
			drawNowDateTime();
			timer.Enabled = true;
			
			//COMポートを探す
			if (comPortListup())
			{
				setControlEnabled();
				cbCOMPortList.Enabled = true;
				btnConnect.Enabled = true;
				info("COMポートを指定し, 接続してください.");
			}
			else
			{
				info("COMポートが見つかりません.");
			}
		}
		
		//about
		private void lblVersion_Click(object sender, EventArgs e)
		{
			lblVersion.Text = APP_ABOUT;
		}
		
		//現在の状態やエラーメッセージを表示する
		public void info(string mes)
		{
			lblInfo.Text = mes;
		}
		
		//COMポートを選択リストにセットする
		private	bool comPortListup()
		{
			cbCOMPortList.Items.Clear();
			foreach (string name in SerialPort.GetPortNames())
				cbCOMPortList.Items.Add(name);

			if (cbCOMPortList.Items.Count == 0)
				return false;
			
			cbCOMPortList.SelectedIndex = cbCOMPortList.Items.Count - 1;
			return true;
		}

		//日時のコントロールのItemsを登録する
		private void setDateControlValue()
		{
			//時分秒
			cbHour.Items.Clear();
			for (int i = 0; i <= 23; i++)
				cbHour.Items.Add(string.Format("{0:d02}", i));

			cbMin.Items.Clear();
			for (int i = 0; i <= 59; i++)
				cbMin.Items.Add(string.Format("{0:d02}", i));

			cbSec.Items.Clear();
			for (int i = 0; i <= 59; i++)
				cbSec.Items.Add(string.Format("{0:d02}", i));
		}
		
		//コントロールのenabled/disabledを設定する
		private void setControlEnabled()
		{
			cbCOMPortList.Enabled = !isConnect;
			btnConnect.Text = isConnect ? "解除" : "接続";
			pnlClockSetting.Enabled = isConnect;
		}

		//接続開始／接続解除
		private void btnConnect_Click(object sender, EventArgs e)
		{
			lblVersion.Text = APP_VER;
			connectAction();
		}
		private void connectAction()
		{
			if (isConnect)
			{
				if (!doDisconnect()) return;
				
				//コントロールの有効状態を設定する
				setControlEnabled();
			}
			else
			{
				if (!doConnect())
				{
					setControlEnabled();
					return;
				}
				
				//シリアル通信初期化（AVR-CDCのRXD/TXDを使う準備）
				if (!cdcio.initSerial())
				{
					//AVR-CDCではないCOMポートへ接続している可能性が高い
					doDisconnect();
					info("ポートが違うか, AVR-CDCが動作していません.");
					return;
				}
				
				//Seg7Clockクラスにcdcioをセットする
				//これ以降、デバイス（時計）を操作してよい
				clockDevice.setCdcIO(cdcio);
				
				//コントロールの有効状態を設定する
				setControlEnabled();
				
				//デバイスの現在の状態を取得する
				if (!getDeviceStatus())
					pnlClockSetting.Enabled = false; 
			}
		}
		
		//接続開始する
		private bool doConnect()
		{
			string	portName = (string)cbCOMPortList.SelectedItem;
			if (!cdcio.Open(portName))
			{
				info(portName + "のオープン失敗.");
				return false;
			}

			isConnect = true;
			info(portName + "に接続しました.");
			return true;
		}
		
		//デバイスの現在の状態を取得する
		private bool getDeviceStatus()
		{
			if (!clockDevice.getValue())
			{
				info("現在値が取得できませんでした.");
				return false;
			}
			
			//値をコントロールに反映させる
			valueToControlDisplayMode();
			valueToControlEffect();
			valueToControlContrast();
			return true;
		}

		//表示モード（時間制）の値をコントロールに反映させる
		private void valueToControlDisplayMode()
		{
			cbDisplayMode.SelectedIndex = Math.Min(clockDevice.dispMode, cbDisplayMode.Items.Count);
		}
		
		//更新エフェクトの値をコントロールに反映させる
		private void valueToControlEffect()
		{
			chkbEffect.Checked = (clockDevice.effect == 1);
		}
		
		//コントラスト（セグメントの明るさ）の値をコントロールに反映させる
		private void valueToControlContrast()
		{
			barContrast.Value = Math.Min(clockDevice.contrast, barContrast.Maximum);
			drawContrastValue();
		}
		
		//接続解除する
		private bool doDisconnect()
		{
			bool	error = false;
			string	portName = (string)cbCOMPortList.SelectedItem;
			
			try
			{
				cdcio.Close();
			}
			catch
			{
				error = true;
				info(portName + "のクローズ失敗.");
			}
			finally
			{
				try
				{
					cdcio.Dispose();
				}
				catch
				{
					if (!error)
					{
						error = true;
						info(portName + "の破棄失敗.");
					}
				}
			}

			isConnect = cdcio.IsOpen;
			if (isConnect)
				error = true;
			else
				info(portName + "を接続解除しました.");
			
			return !error;
		}
		
		//フォームを閉じるとき
		private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
		{
			//接続中だった場合、接続を解除する
			//※アプリを終了すれば自動的に解除されるが、明示的に解除しておく
			if (isConnect) connectAction();
		}
		
		//更新エフェクトの変更
		private void chkbEffect_CheckedChanged(object sender, EventArgs e)
		{
			changeEffect();
		}
		private void changeEffect()
		{
			if (!clockDevice.setConEff(clockDevice.contrast, chkbEffect.Checked ? 1 : 0))
				valueToControlEffect();
		}
		
		//コントラスト変更
		private void barContrast_Scroll(object sender, EventArgs e)
		{
			changeContrast();
		}
		private void changeContrast()
		{
			if (!clockDevice.setConEff(barContrast.Value, clockDevice.effect))
				valueToControlContrast();
			drawContrastValue();
		}

		//コントラストの値を描画する
		private void drawContrastValue()
		{
			lblContrast.Text = "明るさ : " + (barContrast.Value == 0 ? "明るい" : "暗い");
		}

		//表示モード変更
		private void cbDisplayMode_SelectionChangeCommitted(object sender, EventArgs e)
		{
			changeDisplayMode();
		}
		private void changeDisplayMode()
		{
			if (clockDevice.setDisplayMode(cbDisplayMode.SelectedIndex))
				clockDevice.dispMode = cbDisplayMode.SelectedIndex;
			else
				valueToControlDisplayMode();
		}

		//日時設定タイプで「現在の日時」を選択した
		private void rbtnToday_CheckedChanged(object sender, EventArgs e)
		{
			dateReadyToday();
		}
		private void dateReadyToday()
		{
			pnlDate.Enabled = false;
			timer.Enabled = true;
		}
		
		//1秒経過
		private void timer_Tick(object sender, EventArgs e)
		{
			drawNowDateTime();
		}
		
		//現在の時刻をリストボックスに表示する
		private void drawNowDateTime()
		{
			DateTime dt = DateTime.Now;
			cbHour.SelectedIndex = dt.Hour;
			cbMin.SelectedIndex = dt.Minute;
			cbSec.SelectedIndex = dt.Second;
		}
		
		//日時設定タイプで「任意の日時」を選択した
		private void rbtnUser_CheckedChanged(object sender, EventArgs e)
		{
			dateReadyUser();
		}
		private void dateReadyUser()
		{
			pnlDate.Enabled = true;
			timer.Enabled = false;
		}

		//日時を確定して送信する
		private void btnSetDate_Click(object sender, EventArgs e)
		{
			setLcdClockDate();
		}
		private void setLcdClockDate()
		{
			DateTime dt = DateTime.Now;		//← if (rbtnToday.Checked)
			if (rbtnUser.Checked) dt = cbMakeDataTime();

			if (!clockDevice.setDate(dt))
				info("日時の設定に失敗しました.");
		}

		//リストボックスの日時からDateTime構造体を作る
		private DateTime cbMakeDataTime()
		{
			//不正な日付でDateTimeをnewすると例外が発生する
			try
			{
				return new DateTime(
					DateTime.Now.Year,
					DateTime.Now.Month,
					DateTime.Now.Day,
					cbHour.SelectedIndex,
					cbMin.SelectedIndex,
					cbSec.SelectedIndex
				);
			}
			catch { return DateTime.Now; }
		}
		
	}
}
